@@ -1,5 +1,5 @@
 from dataclasses import dataclass, field
-from typing import List, Optional
+from typing import List, Optional, Union
 
 
 @dataclass
@@ -10,6 +10,7 @@ class Allhttp:
 @dataclass
 class NameWrap:
     name: str
+    base: Optional[str] = None
 
 
 @dataclass
@@ -24,6 +25,19 @@ class BasicAuth:
     password: str
 
 
+@dataclass
+class DigestAuth:
+    username: str
+    password: str
+
+
+@dataclass
+class AuthWrap:
+    # digest = DIGESTAUTH | basicauth = BASICAUTH
+    digest_auth: Optional[DigestAuth] = None
+    basic_auth: Optional[BasicAuth] = None
+
+
 @dataclass
 class Query:
     key: str
@@ -81,14 +95,36 @@ class TestScript:
     script: Optional[str]
 
 
+@dataclass
+class Certificate:
+    cert: str
+    key: Optional[str]
+
+
+@dataclass
+class P12Certificate:
+    file: Optional[str] = None
+    password: Optional[str] = None
+
+
+@dataclass
+class ExtraArg:
+    # clears session after each
+    clear: Optional[str] = ''
+    # allows insecure
+    insecure: Optional[str] = ''
+
+
 @dataclass
 class Http:
     namewrap: Optional[NameWrap]
     urlwrap: UrlWrap
-    basic_auth_wrap: Optional[BasicAuth]
+    authwrap: Optional[AuthWrap]
+    certificate: Optional[Certificate]
     lines: Optional[List[Line]]
     payload: Optional[Payload]
     output: Optional[ToFile]
+    extra_args: Optional[ExtraArg] = field(default_factory=lambda: [])
     script_wrap: Optional[TestScript] = field(default_factory=lambda: TestScript(''))
 
 
