@@ -6,8 +6,10 @@ import sys
 from collections import defaultdict
 from dataclasses import dataclass, field
 from io import IOBase
-from typing import Union, List, Optional, Dict, DefaultDict, Tuple, BinaryIO
-from urllib.parse import urlencode
+from typing import Union, List, Optional, Dict, DefaultDict, Tuple, BinaryIO, Any
+from urllib.parse import urlencode, urljoin
+
+from requests.auth import HTTPBasicAuth, HTTPDigestAuth, AuthBase
 
 from .utils import get_real_file_path, triple_or_double_tostring
 
@@ -22,7 +24,8 @@ from textx import TextXSyntaxError, metamodel_from_file
 
 from .dsl_jsonparser import json_or_array_to_json
 from .exceptions import *
-from .parse_models import Allhttp
+from .parse_models import Allhttp, AuthWrap, DigestAuth, BasicAuth, Line, Query, Http, NameWrap, UrlWrap, Header, \
+    MultiPartFile, FilesWrap, TripleOrDouble, Payload as ParsePayload, Certificate, P12Certificate, ExtraArg
 from .property_schema import property_schema
 from .property_util import PropertyProvider
 
@@ -83,9 +86,13 @@ class HttpDef:
     url: str = None
     headers: dict = None
     query: dict = None
-    auth: Tuple[str] = None
+    auth: AuthBase = None
     payload: Optional[Payload] = None
+    certificate: Optional[List[str]] = None
+    p12: Optional[List[str]] = None
     output: str = None
+    allow_insecure = False
+    session_clear = False
     test_script: str = ""
 
     def get_har(self):
@@ -133,6 +140,67 @@ class HttpDef:
     def get_headers(self):
         return [{"name": key, "value": value} for key, value in self.headers.items()] if self.headers else []
 
+    def get_http_from_req(self):
+        data = None
+        datajson = None
+        file = None
+        json_payload = None
+        fileswrap = None
+        type = None
+        if self.payload.filename:
+            file = self.payload.filename
+        elif isinstance(self.payload.data, str):
+            data = [TripleOrDouble(str=self.payload.data)]
+        elif isinstance(self.payload.data, dict):
+            datajson = None
+        elif self.payload.json:
+            json_payload = self.payload.json
+        elif self.payload.files:
+            fileswrap = FilesWrap([])
+            for filekey, multipartdata in self.payload.files:
+                fileswrap.files.append(
+                    MultiPartFile(name=filekey,
+                                  path=multipartdata[1].name if multipartdata[0] else multipartdata[1],
+                                  type=multipartdata[2] if len(multipartdata) > 2 else None)
+                )
+
+        payload = ParsePayload(data=data, datajson=datajson, file=file, json=json_payload, fileswrap=fileswrap,
+                               type=type)
+
+        query_lines = []
+        for key, values in self.query.items():
+            for value in values:
+                query_lines.append(Line(header=None, query=Query(key=key, value=value)))
+        auth_wrap = None
+        if self.auth:
+            if isinstance(self.auth, HTTPBasicAuth):
+                auth_wrap = AuthWrap(basic_auth=BasicAuth(self.auth.username, self.auth.password))
+            elif isinstance(self.auth, HTTPDigestAuth):
+                auth_wrap = AuthWrap(digest_auth=DigestAuth(self.auth.username, self.auth.password))
+        certificate = None
+        if self.certificate:
+            certificate = Certificate(*self.certificate)
+        elif self.p12:
+            certificate = P12Certificate(*self.p12)
+        extra_args = []
+        if self.session_clear:
+            extra_args.append(ExtraArg(clear="@clear"))
+        if self.allow_insecure:
+            extra_args.append(ExtraArg(insecure="@insecure"))
+        return Allhttp(allhttps=[Http(
+            namewrap=NameWrap(self.name),
+            extra_args=extra_args,
+            urlwrap=UrlWrap(url=self.url, method=self.method),
+            lines=[
+                      Line(header=Header(key=key, value=value), query=None)
+                      for key, value in
+                      self.headers.items()] + query_lines
+            ,
+            payload=payload,
+            certificate=certificate,
+            output=None, authwrap=auth_wrap
+        )])
+
 
 @dataclass
 class Property:
@@ -273,25 +341,40 @@ class BaseModelProcessor:
 
     def select_target(self):
         if target := self.args.target:
-            if not isinstance(target, str):
-                target = str(target)
-            if target.isdecimal():
-                if 1 <= int(target) <= len(self.model.allhttps):
-                    self.http = self.model.allhttps[int(target) - 1]
-                else:
-                    raise ParameterException(message="target startswith 1", key='target',
-                                             value=target)
-            else:
-                try:
-                    # if multiple names have same value, it will create confusion
-                    # if they want to go with that. then pass id
-                    self.http = next(filter(lambda http: http.namewrap.name == target,
-                                            (http for http in self.model.allhttps if http.namewrap)))
-                except StopIteration:
-                    raise ParameterException(message="target is not spelled correctly", key='target',
-                                             value=target)
+            self.http = self.get_target(target, self.model.allhttps)
         else:
             self.http = self.model.allhttps[0]
+        self.base_http = None
+        if self.http.namewrap and self.http.namewrap.base:
+            base = self.http.namewrap.base
+            if base == self.http.namewrap.name:
+                raise ParameterException(message="target and base should not be equal", key=target,
+                                         value=base)
+            try:
+                self.base_http = self.get_target(base, self.model.allhttps)
+            except Exception:
+                raise UndefinedHttpToExtend(target=self.http.namewrap.name, base=base)
+
+    @staticmethod
+    def get_target(target: Union[str, int], http_def_list: List[Http]):
+        if not isinstance(target, str):
+            target = str(target)
+        if target.isdecimal():
+            if 1 <= int(target) <= len(http_def_list):
+                selected = http_def_list[int(target) - 1]
+            else:
+                raise ParameterException(message="target startswith 1", key='target',
+                                         value=target)
+        else:
+            try:
+                # if multiple names have same value, it will create confusion
+                # if they want to go with that. then pass id
+                selected = next(filter(lambda http: http.namewrap.name == target,
+                                       (http for http in http_def_list if http.namewrap)))
+            except StopIteration:
+                raise ParameterException(message="target is not spelled correctly", key='target',
+                                         value=target)
+        return selected
 
     def validate_names(self):
         names = []
@@ -337,19 +420,55 @@ class HttpDefBase(BaseModelProcessor):
         """
         headers = {}
         headers.update(self.default_headers)
-        for line in self.http.lines:
-            if header := line.header:
-                self.remove_quotes(header, "'")
-                self.remove_quotes(header, '"')
-                headers[self.get_updated_content(header.key)] = self.get_updated_content(header.value)
+        self.load_headers_to_dict(self.base_http, headers)
+        self.load_headers_to_dict(self.http, headers)
         request_logger.debug(
             f'computed query params from `{self.file}` are `{headers}`')
         self.httpdef.headers = headers
 
+    def load_headers_to_dict(self, http, headers):
+        if not http:
+            return
+        for line in http.lines:
+            if header := line.header:
+                self.remove_quotes(header, "'")
+                self.remove_quotes(header, '"')
+                headers[self.get_updated_content(header.key)] = self.get_updated_content(header.value)
+
+    def load_certificate(self):
+        request_logger.debug(
+            f'url is {self.http.certificate}')
+        certificate: Union[Certificate, P12Certificate] = self.get_current_or_base("certificate")
+        if certificate:
+            if certificate.cert:
+                self.httpdef.certificate = [self.get_updated_content(certificate.cert),
+                                            self.get_updated_content(certificate.key) if certificate.key else None]
+            elif certificate.file:
+                self.httpdef.p12 = [self.get_updated_content(certificate.file),
+                                    self.get_updated_content(certificate.password)]
+
+    def load_extra_flags(self):
+        # flags are extendable
+        # once its marked as allow insecure
+        # user would want all child to have same effect
+        extra_args = self.http.extra_args
+        if self.base_http and self.base_http.extra_args:
+            extra_args += self.base_http.extra_args
+        if extra_args:
+            for flag in extra_args:
+                if flag.clear:
+                    self.httpdef.session_clear = True
+                elif flag.insecure:
+                    self.httpdef.allow_insecure = True
+
     def load_url(self):
         request_logger.debug(
             f'url is {self.http.urlwrap.url}')
-        self.httpdef.url = self.get_updated_content(self.http.urlwrap.url)
+        if base_http := self.base_http:
+            self.httpdef.url = urljoin(self.get_updated_content(base_http.urlwrap.url),
+                                       self.get_updated_content(self.http.urlwrap.url))
+        else:
+            self.httpdef.url = self.get_updated_content(self.http.urlwrap.url)
 
     def load_method(self):
         if method := self.http.urlwrap.method:
@@ -462,9 +581,22 @@ class HttpDefBase(BaseModelProcessor):
             return sys.stdout
 
     def load_auth(self):
-        if auth_wrap := self.http.basic_auth_wrap:
-            self.httpdef.auth = self.get_updated_content(auth_wrap.username), self.get_updated_content(
-                auth_wrap.password)
+        auth_wrap = self.get_current_or_base("authwrap")
+        if auth_wrap:
+            if basic_auth := auth_wrap.basic_auth:
+                self.httpdef.auth = HTTPBasicAuth(self.get_updated_content(basic_auth.username),
+                                                  self.get_updated_content(
+                                                      basic_auth.password))
+            elif digest_auth := auth_wrap.digest_auth:
+                self.httpdef.auth = HTTPDigestAuth(self.get_updated_content(digest_auth.username),
+                                                   self.get_updated_content(
+                                                       digest_auth.password))
+
+    def get_current_or_base(self, attr_key) -> Any:
+        if getattr(self.http, attr_key):
+            return getattr(self.http, attr_key)
+        elif self.base_http:
+            return getattr(self.base_http, attr_key)
 
     def load_def(self):
         if self._loaded:
@@ -476,7 +608,9 @@ class HttpDefBase(BaseModelProcessor):
         self.load_query()
         self.load_payload()
         self.load_auth()
+        self.load_certificate()
         self.load_test_script()
+        self.load_extra_flags()
         self._loaded = True
 
     def load_test_script(self):
